; ============================================
;  Yoshi egg disassembly
;  Disassembled by Ersanio
;  Uses extra bits: NO
;  This egg will retain it's defined color
;  No matter what the X position is!
;  This also checks if a switch palace has
;  been activated yet. Useful for hatching
;  yellow eggs only if you pressed the yellow
;  switch for example.
; ============================================
;  Defines underneath
; ===============================

!YoshiSprite = $2D				; Sprite to spawn when you dont have Yoshi:
						; Currently set to Baby Yoshi

!Alternative = $78				; Sprite to spawn when you already possess over Yoshi
						; Currently set to 1-up

!HatchTime = $20				; How long it takes to hatch the egg.

!1steggpal = $0A				; PPP portion of the egg. AKA Yoshi color.
						; 0A = Green, 08 = Red
						; 06 = Blue,  04 = Yellow

!Palace = $1F27					;$1F27 is the Green Palace
						;$1F28 is the Yellow Palace
						;$1F29 is the Blue Palace
						;$1F2A is the Red Palace

!YoshiPal = $41|!1steggpal			; Properties of the yoshi egg.

!LeftApproach = $20				; How soon to shatter the egg when you approach it from the left
!RiteApproach = $40				; How soon to shatter the egg when you approach it from the right

!HatchSFX = $0A					; Which sound effect to play when egg hatches
!SFXPanel = $1DFC				; Which SFX panel the sound should come from

; ===============================
;  Main Routine
; ===============================

print "INIT ",pc
		INC $187B,X			;Sprite can't be killed by jumping on it.
		RTL				; Return 

YoshiEggTiles:		db $62,$02,$02,$00	;Egg tiles

DATA_01F75C:		db $00,$01,$01,$01	;"Uses next page" flag table during hatching

DATA_01F82D:		db $00,$00,$08,$08	;Spawned minor extended sprite Y position
s

						;Relative to the Yoshi Egg's Bottom
						;(The spawned sprite is the egg shatter)
	
DATA_01F831:		db $00,$08,$00,$08	;Spawned minor extended sprite X position
s

						;Relative to the Yoshi Egg's Bottom
						;(The spawned sprite is the egg shatter)

DATA_01F835:		db $E8,$E8,$F4,$F4	;Y Speed of the spawned shatter
DATA_01F839:		db $FA,$06,$FD,$03	;X speed of the spawned shatter

print "MAIN ",pc
		PHB
		PHK
		PLB
		JSR MainRoutine			;I WONDER WHAT THIS COMMAND DOES.
		PLB
		RTL

MainRoutine:	LDA $187B,X			;\If sprite can be jumped on somehow
		BEQ Label2			;/branch
		JSR SUB_OFF_SCREEN_X0		;\If sprite is offscreen     
		BNE Label0			;/Branch

		LDA !Palace			;\ Change to BNE if you want to hatch the eggs 
		BEQ Label0			;/ BEFORE you hit the selected switch palace

		JSR SUB_HORZ_POS		;Update horizontal position
		LDA $0F				;\
		CLC				; |Load position
		ADC #!LeftApproach             	; |Add #$20 to it
		CMP #!RiteApproach		;/
		BCS Label0			;If the result is greater than #$40 branch
		STZ $187B,X			;Sprite can be jumped on now.
        
		JSL $01F751			;Time to hatch egg and play SFX

		LDA #!HatchTime			; \ Set time to hatch egg.
		STA $1540,X			; /
		LDA #!HatchSFX			; \ Play sound effect (hatch sound)
		STA !SFXPanel			; / 

		LDA #!YoshiSprite		; Sprite number of baby yoshi
		LDY $18E2			; If there is no adult yoshi present
		BEQ Label1			; Go spawn baby yoshi
		LDA #!Alternative		; Sprite number of 1-up
Label1:		STA $151C,X			; Spawn sprite

Label0:		JSR GET_DRAW_INFO		;Set graphics
  
		LDA $00				;\OAM X pos
		STA $0300,Y			;/

		LDA $01				;\OAM Y pos
		STA $0301,Y			;/

		LDA #$00			;\OAM tile
		STA $0302,Y			;/(16x16)

		LDA.b #!YoshiPal
		STA $0303,Y			;Store properties into OAM tile

		LDY #$02			; #$02 means the tiles are 16x16
		LDA #$00			; This means we drew one tile
		JSL $01B7B3			;Finish up OAM routine
		RTS				; Return 



Label2:		LDA $1540,X			;Sprite spinjump death frame counter
		BEQ Label5			;If zero, branch
		LSR				;\
		LSR				; | Sprite spinjump death frame counter >> 3
		LSR				;/

		TAY				;to index
		LDA YoshiEggTiles,y		;\Load egg tiles
		PHA				;/push
		LDA DATA_01F75C,Y		;\Load properties
		PHA				;/push

		JSR GET_DRAW_INFO		;Get draw info

		LDA $00				;\OAM X pos
		STA $0300,Y			;/

		LDA $01				;\OAM Y pos
		STA $0301,Y			;/

		PLA				;\recover property
		STA $00				;/store in temporary buffer

		LDA $0303,Y			;\ Load properties
		AND #$FE			; | Clear everything except the T bit
		ORA $00				; | ORA with the properties from table
		STA $0303,Y			;/ Store into properties

		PLA				;\recover tiles
		STA $0302,Y			;/and store it into OAM tile

		LDY #$02			; #$02 means the tiles are 16x16
		LDA #$00			; This means we drew one tile
		JSL $01B7B3			; Finish up OAm routine
		RTS				; Return 

Label5:		JSR Label4         
		JMP Label6 

Label4:		JSR SUB_OFF_SCREEN_X0		;If offscreen
		BNE Return01F82C		;return

		LDA $E4,X			;Sprite Xpos low
		STA $00				;to 00
		LDA $D8,X			;sprite ypos low
		STA $02				;to 02
		LDA $14D4,X			;Sprite ypos high
		STA $03				;to 03
                  
		PHX				;preserve X
		LDY #$03
		LDX #$0B
Loop:		LDA $17F0,X
		BEQ Label3
Nestedloop:	DEX
		BPL Loop
		DEC $185D
		BPL Anotherloop           
		LDA #$0B                
		STA $185D               
Anotherloop:	LDX $185D               
Label3:		LDA #$03                
		STA $17F0,X

;Set up shatter behaviour

		LDA $00				;\ Load sprite Y low position
		CLC				; |
		ADC DATA_01F831,Y		; |Add Minor extended sprite X position low values from table
		STA $1808,X			;/ Store to extended sprite reg

		LDA $02				;\ Load sprite Y low position
		CLC				; |
		ADC DATA_01F82D,Y		; |Add Minor extended sprite Y position low values from table
		STA $17FC,X			;/Store to extended sprite reg

		LDA $03				;Y pos high of sprite
		STA $1814,X			;To Y pos high of extended sprite

		LDA DATA_01F835,Y		;\Shatter Y speed
		STA $1820,X			;/

		LDA DATA_01F839,Y		;\Shatter X s[eed
		STA $182C,X			;/

		TYA                       
		ASL                       
		ASL                       
		ASL				;do whatever.
		ASL                       
		ASL                       
		ASL                       
		ORA #$28			;\How long the shatters should appear on the screen
		STA $1850,X			;/
		DEY                       
		BPL Nestedloop           
		PLX
Return01F82C:	RTS				; Return 

Label6:		LDA $151C,X			;Load sprite # from somewhere?
		STA $9E,X			;Store into sprite number
		CMP #$35			;If sprite # is 35 (green Yoshi)
		BEQ Label8			;branch
		CMP #$2D			;If it is NOT baby green yoshi
		BNE Label7			;Branch

;Routine for baby green yoshi spawning

		LDA #$09			; \ Sprite status = Carryable 
		STA $14C8,X			; / 
		LDA.b #!YoshiPal
		AND #$0E			;Preserve the palette bits
		PHA				;Push A
		JSL $07F7D2			;init Sprite tables
		PLA				;recover A/palette bits
		STA $00				;store palette bits into $00	
		LDA $15F6,X			;Load sprite properties
		AND #$F1			;Clear everything except palette bits
		ORA $00				;ORA with the palette bits
		STA $15F6,X			;Update sprite properties
		RTS				; Return

Label7:		JSL $07F7D2			;init Sprite tables
		RTS				; *yawn*

;ROUTINE TO SPAWN AN ADULT YOSHI AFTER EGG APPEARS
;FROM A [?] BLOCK I GUESS?
;USELESS ROUTINE SO I COMMENTED IT OUT.

Label8:		;JSL $07F7D2			;init Sprite tables
;
;		STZ $18AC
;		STZ $1E14			; No Yoshi wings 
;		LDA #$35			; \ Sprite = Yoshi 
;		STA $9E,X			; / 
;		LDA #$08			; \ Sprite status = Normal 
;		STA $14C8,X			; / 
;		LDA #$1F			; \
;		STA $1DFC			; / Play sound effect 
;		LDA $D8,X
;		SBC #$10
;		STA $D8,X
;		LDA $14D4,X
;		SBC #$00
;		STA $14D4,X
;		LDA $15F6,X
;		PHA				; \ Reset sprite tables 
;		JSL $07F7D2			;  |
;		PLA				; / 
;		AND #$FE                
;		STA $15F6,X     
;		LDA #$0C                
;		STA $1602,X             
;		DEC $160E,X             
;		LDA #$40			;Set Yoshi Growing Animation Timer
;		STA $18E8			;To max
		RTS				; Return 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:            db $40,$B0
SPR_T13:            db $01,$FF
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		    db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
                    db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:  LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:  LDA #$0E                ;  |
STORE_03:           STA $03                 ;  |            
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_X0:  STZ $03                 ; /

START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return

VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:  LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS:		LDY #$00
			LDA $94
			SEC
			SBC $E4,x
			STA $0F
			LDA $95
			SBC $14E0,x
			BPL SPR_L16
			INY
SPR_L16:		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:             db $0C,$1C
SPR_T2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC #$0040              ;  |
                    CMP #$0180              ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
INVALID:            RTS                     ; return